module.exports = {
  props: {
    classes: {
      type: Array,
      "default": function() {
        return ["table-of-contents"];
      }
    }
  },
  mixins: [require("vue-mixins/onWindowScroll"), require("vue-mixins/getViewportSize"), require("vue-mixins/onWindowResize"), require("vue-mixins/getDocumentHeight")],
  data: function() {
    return {
      height: this.getViewportSize().height,
      targets: [],
      names: [],
      lastScrollTop: 0,
      scrollTop: 0,
      activated: false
    };
  },
  events: {
    "addTarget": function(targetName, targetEl) {
      this.targets.push(targetEl);
      this.names.push(targetName);
      return true;
    },
    "removeTarget": function(targetName) {
      var index;
      index = this.names.indexOf(targetName);
      this.targets.splice(index, 1);
      this.names.splice(index, 1);
      return true;
    },
    "activate": function(targetName) {
      this.activated = true;
      this.$broadcast("activate", targetName);
      return true;
    }
  },
  methods: {
    processScroll: function(e) {
      var area, bottom, coverage, el, i, indices, j, len, max, name, rect, ref, top;
      if (this.activated) {
        return this.activated = false;
      } else {
        this.lastScrollTop = this.scrollTop;
        this.scrollTop = window.pageYOffset || document.documentElement.scrollTop;
        if (this.scrollTop + this.height - this.getDocumentHeight() > -40) {
          name = this.names[this.names.length - 1];
        } else {
          coverage = [];
          indices = [];
          ref = this.targets;
          for (i = j = 0, len = ref.length; j < len; i = ++j) {
            el = ref[i];
            rect = el.getBoundingClientRect();
            if (this.lastScrollTop <= this.scrollTop) {
              top = Math.max(0 + this.height * 15 / 50, rect.top);
              bottom = Math.min(this.height * 20 / 50, rect.bottom);
            } else {
              top = Math.max(0 + this.height * 30 / 50, rect.top);
              bottom = Math.min(this.height * 35 / 50, rect.bottom);
            }
            area = bottom - top;
            if (area >= 0) {
              indices.push(i);
              coverage.push(area);
            }
          }
          max = coverage.reduce((function(iMax, x, i, a) {
            if (x > a[iMax]) {
              return i;
            } else {
              return iMax;
            }
          }), 0);
          name = this.names[indices[max]];
        }
        return this.$broadcast("activate", name);
      }
    }
  },
  compiled: function() {
    this.onWindowScroll(this.processScroll);
    return this.onWindowResize((function(_this) {
      return function() {
        return _this.height = _this.getViewportSize().height;
      };
    })(this));
  },
  ready: function() {
    return this.processScroll();
  }
};

if (module.exports.__esModule) module.exports = module.exports.default
;(typeof module.exports === "function"? module.exports.options: module.exports).template = "<ul v-bind:style=style v-bind:class=classes><slot></slot></ul>"
